{ lib     ? import <nixpkgs/lib>
, pkgsFun ? import <nixpkgs>
, hostPlatform ? null         # if left `null` this will be set based on the ./platform/
}:
let
  nixpkgsArgs = { config.allowNonSource = false; };
  nixpkgsOnBuildForBuild  = pkgsFun nixpkgsArgs;

  ownerboot = { overlays }: let
    base = final: prev: {
      inherit hostPlatform;
      nixpkgsOnBuildForHost = pkgsFun
        (nixpkgsArgs
         // (lib.optionalAttrs (final.hostPlatform!=null) {
           crossSystem = final.hostPlatform;
         }));

      coreboot-toolchain = final.callPackage ./coreboot-toolchain { };

      # TODO(amjoseph): move this into a `config` attrset
      console-device = throw "platforms must override this (example: \"ttyS0\")";
      console-parameters = "115200n8";
      linux-command-line =
        let console = "${final.console-device},${final.console-parameters}";
        in "console=${console} earlyprintk=${console}";

      iasl_20180531 = final.nixpkgsOnBuildForBuild.callPackage ./coreboot/iasl_20180531 { };
      coreboot      = final.callPackage ./coreboot {
        initramfs_image = final.initramfs;
      };
      kernel        = final.callPackage ./kernel { };
      userspace     = final.callPackage ./userspace { };
      initramfs     = final.callPackage ./initramfs { };
      flashrom      = final.nixpkgsOnBuildForHost.callPackage ./util/flashrom { };
    };
  in
    (lib.makeScope
      lib.callPackageWith
      (self: { inherit lib nixpkgsOnBuildForBuild; })).overrideScope'
      (lib.foldl' lib.composeExtensions (_: prev: prev) ([base] ++ overlays));

  common_amd64 = import ./platform/common/amd64.nix { inherit lib; };
  common_arm64 = import ./platform/common/arm64.nix { inherit lib; };

in {
  # one entry for each supported platform
  kevin = ownerboot (import ./platform/kevin { inherit lib common_arm64; });
  am1i  = ownerboot (import ./platform/am1i { inherit lib common_amd64; });
  kgpe  = ownerboot (import ./platform/kgpe { inherit lib common_amd64; });

  em100 = nixpkgsOnBuildForBuild.callPackage ./util/em100 { };
}


